# BASED ON PYPLACE - ALMOST COMPLETE

import data
from cyclepath import CyclePath

from pygame import Rect

DEBUG = True



class GameObject:

    def __init__(self, game_objects):
        self.game_objects = game_objects



class MapObject(GameObject):

    def __init__(self, game_objects, sizes):
        GameObject.__init__(self, game_objects)
        
        self.sizes = sizes
        self.viewport = None
        self.tiles = {}
        
        self.next_actor_id = 1
        self.actors_at = {}
        self.actors = {}
        
        
    def set_viewport(self, viewport):
        self.viewport = viewport
        
        
    def set_tile(self, x, y, images):
        tup_cord = x, y
        if tup_cord not in self.tiles:
            self.tiles[tup_cord] = {"floor": None}
        self.tiles[tup_cord].update(images)


    def height(self, index):
        tile = self.tiles.get(index, None)
        if tile:
            return tile["floor"].height
        else:
            return 0 # @@@ ?         


    def at(self, index): # @@@ rename
        tile = self.tiles.get(index, None)
        if tile:
            return (tile, tile["floor"], tile["floor"].height, tile["pixref"])
        else:
            return (None, self.game_objects.get_floor("void"), 0, 0) # @@@


    def actor(self, actor_id):
        return self.actors.get(actor_id, None)
        
    
    def add_new_actor(self, coord, vals):
        new_id = self.next_actor_id
        self.next_actor_id += 1
        vals["coord"] = coord
        self.actors[new_id] = vals
        self.actors_at.setdefault(coord, []).append(new_id)


    def move_actor(self, actor_id, pos=(0,0), screen=True, fine_pos=0):
        actor = self.actor(actor_id)
        old_coord = actor.get("coord", None)
        actors_at_coord = self.actors_at.get(old_coord, None)
        prop = actor.get("prop", None)
        if screen:
            (converted, move_to_coord, move_to_fine) = self.viewport.screen_to_map(pos, fine_pos)
            if converted:
                pos = move_to_coord
                (prop.mx, prop.my) = move_to_fine
        self.actors_at[old_coord] = filter(lambda x: x != actor_id, actors_at_coord)
        self.actors_at.setdefault(pos, []).append(actor_id)
        self.actors[actor_id]["coord"] = pos
        prop.moved = True

    # @@@ cyclepath

    def remove_actor(self, actor_id):
        old_coord = self.actors[actor_id]["coord"]
        current_place_list = self.actors_at[old_coord]
        new_list = filter(lambda x: x != actor_id, current_place_list)
        self.actors_at[old_coord] = new_list
        del self.actors[actor_id]
        
        
    def turn(self):
        for actor_id, actor in self.actors.items():
            coord = actor["coord"]
            prop = actor["prop"]
            new_coord = prop.do_action(coord, actor)
            if new_coord and new_coord != coord:
                self.move_actor(actor_id, new_coord, screen=False)
        return Rect(0, 0, 0, 0)



class ImagedObject(GameObject):


    def __init__(self, game_objects):

        GameObject.__init__(self, game_objects)

        self.__dict__.update(self.default_data)
        self.init_data()
        self.finalize_data()
        self.pix = {}
        self.load_graphics()
        self.finalize_graphics()
        if not hasattr(self.__class__, "name"):
            self.__class__.name = "noname"
        
        
    def finalize_data(self):
        pass
        
        
    def finalize_graphics(self):
        pass
        
        
    def __getitem__(self, key):
        return self.pix[key]
        
        

class FloorTile(ImagedObject):
    default_data = {
        "quick_tiles": False,
    }



class PropTile(ImagedObject):
    default_data = {}



class ActorTile(PropTile):
    default_data = {
        "mx": 8,
        "my": 8,
        "ix": 0,
        "iy": 0,
        "fine_steps": 2,
        "moved": True
    }
    
    def __init__(self, game_objects):
        
        self.cycle_path = CyclePath(self)
        self.cycle_path.set_state(peek_steps=4, can_climb=3)

        # @@@ this could be first if init_data was called directly
        
        PropTile.__init__(self, game_objects)
        
    def msg(self, place, action=None, **params):
        self.cycle_path.msg(place, action, **params)
    

    def ride_cycle(self, coords):
        return self.cycle_path.ride(coords)
                
       

class GameObjects:

    def __init__(self):
        self.background_objects = {}
        self.floor_objects = {}
        self.prop_objects = {}
        

    def load_backgrounds(self, name):
        _load_objects(self.background_objects, name)


    def load_floor(self, name):
        _load_objects(self.floor_objects, name)


    def load_props(self, name):
        _load_objects(self.prop_objects, name)
        
         
    def get_background(self, object_name):
        return self.background_objects[object_name](self)


    def get_floor(self, object_name):
        return self.floor_objects[object_name](self)


    def get_prop(self, object_name):
        return self.prop_objects[object_name](self)
        
        

def _load_objects(container, name):
    wildcard = "*_" + name + ".py"
    base = name + "s"
    files = data.find(wildcard, base)
    for f in files:
        if DEBUG: print "Loading file", f
        objs = data.fetch_objects(f)
        container.update(objs)