# BASED ON PYPLACE - ALMOST COMPLETE

import re

import pygame.display

from gameobj import MapObject


# @@@
SCREEN_SIZE = (800, 600)


chartypes = {
    "v": "void",
    "g": "grass",
    "c": "cement",
    "C": "curb",
}

class Level(MapObject):


    def __init__(self, surface, game_objects):
    
        MapObject.__init__(self, game_objects, (100, 100, 1))

        self.surface = surface
        self.scroll_allowed = True


    
    def build(self, coord, map_text):
    
        (row, col_base) = coord
        
        for line in map_text.split("="):
            col = col_base
            line = re.sub(r"[^ -z]", "", line)
            for t in line:
                name = chartypes[t]
                floor_tile = self.game_objects.get_floor(name)
                # @@@ no handling of random
                self.set_tile(col, row, {"floor": floor_tile, "pixref": 0})
                col += 1
            row += 1
            
        
    def load_background(self, object_name, image_key):
        bg_img = self.game_objects.get_background(object_name)[image_key]
        bg_rect = bg_img.get_rect()
        (bg_maxx, bg_maxy) = bg_img.get_size()
        bg_rect.bottomleft = (0, SCREEN_SIZE[1] - 1)
        self.surface.blit(bg_img, bg_rect)
        pygame.display.update(bg_rect)
        
        
    def add_actor(self, coord, actor_id, prop_class):
        prop_tile = self.game_objects.get_prop(prop_class)
        # @@@ no handling of random
        actor = {"prop": prop_tile}
        
        # @@@ actor_id ignored
        self.add_new_actor(coord, actor)        
    

    def msg(self, action=None, group_id=None, **params):
        if group_id and action:
            group_id = int(group_id)
            if action == "JMP": # @@@ ???
                self.move_actor_to(group_id, **params)
            else:
                actor = self.actor(group_id)
                if actor:
                    prop = actor.get("prop", None) # @@@ should just make attr
                    if prop:
                        prop.msg(self, action, **params)
                        
            
    def __render__(self):
        return self.my_widget.__render__(self.surface)
        
        
    def __onclick__(self, mousepos):
        return self.my_widget.__onclick__(mousepos)
        
        
    def __unclick__(self, mousepos):
        return self.my_widget.__unclick__(mousepos)
