# BASED ON PYPLACE - PRETTY MUCH DONE

from pygame import Rect

import widget_graphics

class Widget:
    
    box = []
    
    def __init__(self, rect, needs_render=True):
        self.rect = rect
        self.needs_render = needs_render
        self.unclick_me = False
        Widget.box.append(self)


    def __render__(self, surface):
        rects = []
        save_clip = surface.get_clip()
        for w in Widget.box:
            if w.needs_render:
                surface.set_clip(w.rect)
                rects.append(w.render(surface))
                w.needs_render = False
        surface.set_clip(save_clip)
        
        return rects


    def __onclick__(self, mousepos):
        for w in Widget.box:
            if w.rect.collidepoint(mousepos):
                w.onclick(mousepos)
                

    def __unclick__(self, mousepos):
        for w in Widget.box:
            if w.unclick_me or w.rect.collidepoint(mousepos):
                w.unclick(mousepos)


    def onclick(self, mousepos):
        pass # default
                
                
    def unclick(self, mousepos):
        pass # default



wpix = None
    
class Button(Widget):


    def __init__(self, rect, button_name, init_state=0, max_state=3, step_state=1, action=None, button_id=None):
        self.button_name = button_name
        image_list = []
        loop_state = init_state

        # @@@ improve this
        global wpix
        if not wpix:
            wpix = widget_graphics.load()

        while True:
            lookup = button_name + "_" + str(loop_state)
            if lookup in wpix:
                loop_state += step_state
            else:
                break
            self.image_list = image_list
            self.state = init_state
            self.max_state = max_state
            self.step_state = step_state
            self.action = action
            self.button_id = button_id
            Widget.__init__(self, rect)
    
    
    def set_to(self, new_state):
        if new_state in range(3):
            self.state = new_state
            self.needs_render = True
            
            
    def render(self, surface):
        return self._render_at(surface, (self.rect[0], self.rect[1] + self.rect[3]))
    
    
    def _render_at(self, surface, surpos):
        lookup = self.button_name + "_" + str(self.state)
        img = wpix.get(lookup, None)
        if img:
            r = img.get_rect()
            r.bottomleft = surpos
            surface.blit(img, r)
            return r


    def onclick(self, mousepos):
        self.set_to(1) # @@@ use constants
        self.unclick_me = True
        

    def unclick(self, mousepos):
        self.set_to(0) # @@@ use constants
        if self.unclick_me:
            self.unclick_me = False
            if self.action:
                self.action(self.button_id)    
    

class DirectionButtons(Widget):
    
    def __init__(self, rect, button_stem, num_directions=4, action=None, group_id=None):
        (base_x, base_y, w, h) = rect
        
        self.action = action
        self.group_id = group_id
        
        (tiling_xstep, tiling_ystep) = (32, 32)
        (tile_w, tile_h) = (32, 32)
        
        if num_directions == 8:
            tilesat = ["DR", "D", "DL", "R", "A", "L", "UR", "U", "UL"]
        else:
            tilesat = ["DR", "x", "DL", "x", "A", "x", "UR", "x", "UL"]
            
        for rows_y in range(3):
            for rows_x in range(3):
                butrect = Rect(base_x + rows_x * tiling_xstep, base_y + rows_y * tiling_ystep, tile_w, tile_h)
                direction = tilesat.pop()
                button = Button(butrect, button_stem + direction, action=self.clicked, button_id=direction)
                
        Widget.__init__(self, rect, needs_render=False)
        
        
    def clicked(self, button_id):
        if self.action:
            self.action(self.group_id, button_id)